package cn.bunny.services.service.financial.impl;

import cn.bunny.common.service.context.BaseContext;
import cn.bunny.common.service.exception.BunnyException;
import cn.bunny.dao.dto.financial.bill.admin.BillAddDto;
import cn.bunny.dao.dto.financial.bill.admin.BillDto;
import cn.bunny.dao.dto.financial.bill.admin.BillUpdateDto;
import cn.bunny.dao.dto.financial.bill.user.BillUserAddDto;
import cn.bunny.dao.dto.financial.bill.user.BillUserDto;
import cn.bunny.dao.dto.financial.bill.user.BillUserUpdateDto;
import cn.bunny.dao.entity.financial.Bill;
import cn.bunny.dao.pojo.result.PageResult;
import cn.bunny.dao.pojo.result.ResultCodeEnum;
import cn.bunny.dao.vo.financial.admin.BillVo;
import cn.bunny.dao.vo.financial.user.BillUserVo;
import cn.bunny.services.mapper.financial.BillMapper;
import cn.bunny.services.service.financial.BillService;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import jakarta.validation.Valid;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * <p>
 * 账单信息 服务实现类
 * </p>
 *
 * @author Bunny
 * @since 2024-11-07 16:49:19
 */
@Service
public class BillServiceImpl extends ServiceImpl<BillMapper, Bill> implements BillService {

    /**
     * * 账单信息 服务实现类
     *
     * @param pageParams 账单信息分页查询page对象
     * @param dto        账单信息分页查询对象
     * @return 查询分页账单信息返回对象
     */
    @Override
    public PageResult<BillUserVo> getUserBillList(Page<Bill> pageParams, BillUserDto dto) {
        // 需要根据当前用户去查询
        Long userId = BaseContext.getUserId();

        // 判断创建用户的Id是否和当前请求的用户id是否相同
        IPage<BillUserVo> page = baseMapper.selectUserListByPage(pageParams, dto, userId);

        return PageResult.<BillUserVo>builder()
                .list(page.getRecords())
                .pageNo(page.getCurrent())
                .pageSize(page.getSize())
                .total(page.getTotal())
                .build();
    }

    /**
     * 添加账单信息
     *
     * @param dto 账单信息添加
     */
    @Override
    public void addUserBill(@Valid BillUserAddDto dto) {
        // 保存数据
        Bill bill = new Bill();
        BeanUtils.copyProperties(dto, bill);

        // 设置当前的用户Id
        bill.setUserId(BaseContext.getUserId());
        save(bill);
    }

    /**
     * 更新账单信息
     *
     * @param dto 账单信息更新
     */
    @Override
    public void updateUserBill(@Valid BillUserUpdateDto dto) {
        // 判断当前用户修改的内容是否是自己的内容
        Bill bill = getOne(Wrappers.<Bill>lambdaQuery().eq(Bill::getId, dto.getId()));
        if (!bill.getUserId().equals(BaseContext.getUserId())) {
            throw new BunnyException(ResultCodeEnum.ILLEGAL_DATA_REQUEST);
        }

        // 更新内容
        bill = new Bill();
        BeanUtils.copyProperties(dto, bill);
        updateById(bill);
    }

    /**
     * 删除|批量删除账单信息
     *
     * @param ids 删除id列表
     */
    @Override
    public void deleteUserBill(List<Long> ids) {
        // 判断删除的是否是自己的账单
        List<Bill> billList = list(Wrappers.<Bill>lambdaQuery().in(Bill::getId, ids))
                .stream().filter(bill -> !bill.getUserId().equals(BaseContext.getUserId())).toList();
        if (!billList.isEmpty()) {
            throw new BunnyException(ResultCodeEnum.ILLEGAL_DATA_REQUEST);
        }
        baseMapper.deleteBatchIdsWithPhysics(ids);
    }

    /**
     * 分页查询账单信息
     *
     * @return 账单信息返回列表
     */
    @Override
    public PageResult<BillVo> getBillList(Page<Bill> pageParams, BillDto dto) {
        // 判断创建用户的Id是否和当前请求的用户id是否相同
        IPage<BillVo> page = baseMapper.selectListByPage(pageParams, dto);

        return PageResult.<BillVo>builder()
                .list(page.getRecords())
                .pageNo(page.getCurrent())
                .pageSize(page.getSize())
                .total(page.getTotal())
                .build();
    }

    /**
     * 添加账单信息
     *
     * @param dto 添加表单
     */
    @Override
    public void addBill(BillAddDto dto) {
        // 保存数据
        Bill bill = new Bill();
        BeanUtils.copyProperties(dto, bill);
        save(bill);
    }

    /**
     * 更新账单信息
     *
     * @param dto 更新表单
     */
    @Override
    public void updateBill(BillUpdateDto dto) {
        // 更新内容
        Bill bill = new Bill();
        BeanUtils.copyProperties(dto, bill);
        updateById(bill);
    }

    /**
     * 删除账单信息
     *
     * @param ids 删除的ids
     */
    @Override
    public void deleteBill(List<Long> ids) {
        baseMapper.deleteBatchIdsWithPhysics(ids);
    }
}
